@@ -24,7 +24,6 @@ this module can be modified.
 from __future__ import division, print_function, absolute_import
 
 import os
-import inspect
 import warnings
 import threading
 from enum import Enum, IntEnum, unique
@@ -32,7 +31,6 @@ from enum import Enum, IntEnum, unique
 import attr
 
 from hypothesis.errors import InvalidArgument, HypothesisDeprecationWarning
-from hypothesis.configuration import hypothesis_home_dir
 from hypothesis.internal.compat import text_type
 from hypothesis.utils.conventions import UniqueIdentifier, not_set
 from hypothesis.internal.reflection import proxies, \
@@ -51,9 +49,6 @@ unlimited = UniqueIdentifier('unlimited')
 all_settings = {}
 
 
-_db_cache = {}
-
-
 class settingsProperty(object):
 
     def __init__(self, name, show_default):
@@ -65,7 +60,14 @@ class settingsProperty(object):
             return self
         else:
             try:
-                return obj.__dict__[self.name]
+                result = obj.__dict__[self.name]
+                # This is a gross hack, but it preserves the old behaviour that
+                # you can change the storage directory and it will be reflected
+                # in the default database.
+                if self.name == 'database' and result is not_set:
+                    from hypothesis.database import ExampleDatabase
+                    result = ExampleDatabase(not_set)
+                return result
             except KeyError:
                 raise AttributeError(self.name)
 
@@ -121,28 +123,30 @@ class settings(settingsMeta('settings', (object,), {})):
     """
 
     _WHITELISTED_REAL_PROPERTIES = [
-        '_database', '_construction_complete', 'storage'
+        '_construction_complete', 'storage'
     ]
     __definitions_are_locked = False
     _profiles = {}
 
     def __getattr__(self, name):
         if name in all_settings:
-            d = all_settings[name].default
-            if inspect.isfunction(d):
-                d = d()
-            return d
+            return all_settings[name].default
         else:
             raise AttributeError('settings has no attribute %s' % (name,))
 
-    def __init__(
-            self,
-            parent=None,
-            **kwargs
-    ):
+    def __init__(self, parent=None, **kwargs):
+        if (
+            kwargs.get('database', not_set) is not_set and
+            kwargs.get('database_file', not_set) is not not_set
+        ):
+            if kwargs['database_file'] is None:
+                kwargs['database'] = None
+            else:
+                from hypothesis.database import ExampleDatabase
+                kwargs['database'] = ExampleDatabase(kwargs['database_file'])
+        if not kwargs.get('perform_health_check', True):
+            kwargs['suppress_health_check'] = HealthCheck.all()
         self._construction_complete = False
-        self._database = kwargs.pop('database', not_set)
-        database_file = kwargs.get('database_file', not_set)
         deprecations = []
         defaults = parent or settings.default
         if defaults is not None:
@@ -156,12 +160,10 @@ class settings(settingsMeta('settings', (object,), {})):
                     if setting.validator:
                         kwargs[setting.name] = setting.validator(
                             kwargs[setting.name])
-            if self._database is not_set and database_file is not_set:
-                self._database = defaults.database
         for name, value in kwargs.items():
             if name not in all_settings:
                 raise InvalidArgument(
-                    'Invalid argument %s' % (name,))
+                    'Invalid argument: %r is not a valid setting' % (name,))
             setattr(self, name, value)
         self.storage = threading.local()
         self._construction_complete = True
@@ -237,9 +239,9 @@ class settings(settingsMeta('settings', (object,), {})):
 
     @classmethod
     def define_setting(
-        cls, name, description, default, options=None, deprecation=None,
+        cls, name, description, default, options=None,
         validator=None, show_default=True, future_default=not_set,
-        deprecation_message=None,
+        deprecation_message=None, hide_repr=not_set,
     ):
         """Add a new setting.
 
@@ -262,9 +264,12 @@ class settings(settingsMeta('settings', (object,), {})):
         if future_default is not_set:
             future_default = default
 
+        if hide_repr is not_set:
+            hide_repr = bool(deprecation_message)
+
         all_settings[name] = Setting(
             name, description.strip(), default, options, validator,
-            future_default, deprecation_message,
+            future_default, deprecation_message, hide_repr,
         )
         setattr(settings, name, settingsProperty(name, show_default))
 
@@ -275,12 +280,6 @@ class settings(settingsMeta('settings', (object,), {})):
     def __setattr__(self, name, value):
         if name in settings._WHITELISTED_REAL_PROPERTIES:
             return object.__setattr__(self, name, value)
-        elif name == 'database':
-            assert self._construction_complete
-            raise AttributeError(
-                'settings objects are immutable and may not be assigned to'
-                ' after construction.'
-            )
         elif name in all_settings:
             if self._construction_complete:
                 raise AttributeError(
@@ -304,32 +303,11 @@ class settings(settingsMeta('settings', (object,), {})):
 
     def __repr__(self):
         bits = []
-        for name in all_settings:
+        for name, setting in all_settings.items():
             value = getattr(self, name)
-            bits.append('%s=%r' % (name, value))
-        bits.sort()
-        return 'settings(%s)' % ', '.join(bits)
-
-    @property
-    def database(self):
-        """An ExampleDatabase instance to use for storage of examples. May be
-        None.
-
-        If this was explicitly set at settings instantiation then that
-        value will be used (even if it was None). If not and the
-        database_file setting is not None this will be lazily loaded as
-        an ExampleDatabase, using that file the first time that this
-        property is accessed on a particular thread.
-        """
-        if self._database is not_set and self.database_file is not None:
-            from hypothesis.database import ExampleDatabase
-            if self.database_file not in _db_cache:
-                _db_cache[self.database_file] = (
-                    ExampleDatabase(self.database_file))
-            return _db_cache[self.database_file]
-        if self._database is not_set:
-            self._database = None
-        return self._database
+            if value != setting.default or not setting.hide_repr:
+                bits.append('%s=%r' % (name, value))
+        return 'settings(%s)' % ', '.join(sorted(bits))
 
     def __enter__(self):
         default_context_manager = default_variable.with_value(self)
@@ -402,15 +380,19 @@ class Setting(object):
     validator = attr.ib()
     future_default = attr.ib()
     deprecation_message = attr.ib()
+    hide_repr = attr.ib()
 
 
 settings.define_setting(
     'min_satisfying_examples',
-    default=5,
+    default=not_set,
     description="""
-Raise Unsatisfiable for any tests which do not produce at least this many
-values that pass all assume() calls and which have not exhaustively covered the
-search space.
+This doesn't actually do anything, but remains for compatibility reasons.
+""",
+    deprecation_message="""
+The min_satisfying_examples setting has been deprecated and disabled, due to
+overlap with the filter_too_much healthcheck and poor interaction with the
+max_examples setting.
 """
 )
 
@@ -425,11 +407,13 @@ counter-example, falsification will terminate.
 
 settings.define_setting(
     'max_iterations',
-    default=1000,
+    default=not_set,
     description="""
-Once this many iterations of the example loop have run, including ones which
-failed to satisfy assumptions and ones which produced duplicates, falsification
-will terminate.
+This doesn't actually do anything, but remains for compatibility reasons.
+""",
+    deprecation_message="""
+The max_iterations setting has been disabled, as internal heuristics are more
+useful for this purpose than a user setting.  It no longer has any effect.
 """
 )
 
@@ -472,6 +456,7 @@ limit - Hypothesis won't e.g. interrupt execution of the called
 function to stop it. If this value is <= 0 then no timeout will be
 applied.
 """,
+    hide_repr=False,  # Still affects behaviour at runtime
     deprecation_message="""
 The timeout setting is deprecated and will be removed in a future version of
 Hypothesis. To get the future behaviour set ``timeout=hypothesis.unlimited``
@@ -500,14 +485,9 @@ settings.define_setting(
     'strict',
     default=os.getenv('HYPOTHESIS_STRICT_MODE') == 'true',
     description="""
-If set to True, anything that would cause Hypothesis to issue a warning will
-instead raise an error. Note that new warnings may be added at any time, so
-running with strict set to True means that new Hypothesis releases may validly
-break your code.  Note also that, as strict mode is itself deprecated,
-enabling it is now an error!
-
-You can enable this setting temporarily by setting the HYPOTHESIS_STRICT_MODE
-environment variable to the string 'true'.
+Strict mode has been deprecated in favor of Python's standard warnings
+controls.  Ironically, enabling it is therefore an error - it only exists so
+that users get the right *type* of error!
 """,
     deprecation_message="""
 Strict mode is deprecated and will go away in a future version of Hypothesis.
@@ -517,18 +497,49 @@ warnings.simplefilter('error', HypothesisDeprecationWarning).
     future_default=False,
 )
 
+
+def _validate_database(db, __from_db_file=False):
+    from hypothesis.database import ExampleDatabase
+    if db is None or isinstance(db, ExampleDatabase):
+        return db
+    if __from_db_file or db is not_set:
+        return ExampleDatabase(db)
+    raise InvalidArgument(
+        'Arguments to the database setting must be None or an instance of '
+        'ExampleDatabase.  Try passing database=ExampleDatabase(%r), or '
+        'construct and use one of the specific subclasses in '
+        'hypothesis.database' % (db,)
+    )
+
+
 settings.define_setting(
-    'database_file',
-    default=lambda: (
-        os.getenv('HYPOTHESIS_DATABASE_FILE') or
-        os.path.join(hypothesis_home_dir(), 'examples')
-    ),
+    'database',
+    default=not_set,
     show_default=False,
     description="""
-    An instance of hypothesis.database.ExampleDatabase that will be
+An instance of hypothesis.database.ExampleDatabase that will be
 used to save examples to and load previous examples from. May be None
-in which case no storage will be used.
-"""
+in which case no storage will be used, `:memory:` for an in-memory
+database, or any path for a directory-based example database.
+""",
+    validator=_validate_database,
+)
+
+settings.define_setting(
+    'database_file',
+    default=not_set,
+    show_default=False,
+    description="""
+The file or directory location to save and load previously tried examples;
+`:memory:` for an in-memory cache or None to disable caching entirely.
+""",
+    validator=lambda f: _validate_database(f, __from_db_file=True),
+    deprecation_message="""
+The `database_file` setting is deprecated in favor of the `database`
+setting, and will be removed in a future version.  It only exists at
+all for complicated historical reasons and you should just use
+`database` instead.
+""",
 )
 
 
@@ -547,6 +558,14 @@ class HealthCheck(Enum):
     Each member of this enum is a type of health check to suppress.
     """
 
+    def __repr__(self):
+        return '%s.%s' % (self.__class__.__name__, self.name)
+
+    @classmethod
+    def all(cls):
+        bad = (HealthCheck.exception_in_generation, HealthCheck.random_module)
+        return [h for h in list(cls) if h not in bad]
+
     exception_in_generation = 0
     """Deprecated and no longer does anything. It used to convert errors in
     data generation into FailedHealthCheck error."""
@@ -590,66 +609,39 @@ class Statistics(IntEnum):
     always = 2
 
 
-class Verbosity(object):
+@unique
+class Verbosity(IntEnum):
+    quiet = 0
+    normal = 1
+    verbose = 2
+    debug = 3
+
+    @staticmethod
+    def _get_default():
+        var = os.getenv('HYPOTHESIS_VERBOSITY_LEVEL')
+        if var is not None:  # pragma: no cover
+            try:
+                result = Verbosity[var]
+            except KeyError:
+                raise InvalidArgument('No such verbosity level %r' % (var,))
+
+            warnings.warn(HypothesisDeprecationWarning(
+                'The HYPOTHESIS_VERBOSITY_LEVEL environment variable is '
+                'deprecated, and will be ignored by a future version of '
+                'Hypothesis.  Configure your verbosity level via a '
+                'settings profile instead.'
+            ))
+            return result
+        return Verbosity.normal
 
     def __repr__(self):
         return 'Verbosity.%s' % (self.name,)
 
-    def __init__(self, name, level):
-        self.name = name
-        self.level = level
-
-    def __eq__(self, other):
-        return isinstance(other, Verbosity) and (
-            self.level == other.level
-        )
-
-    def __ne__(self, other):
-        return not self.__eq__(other)
-
-    def __hash__(self):
-        return self.level
-
-    def __lt__(self, other):
-        return self.level < other.level
-
-    def __le__(self, other):
-        return self.level <= other.level
-
-    def __gt__(self, other):
-        return self.level > other.level
-
-    def __ge__(self, other):
-        return self.level >= other.level
-
-    @classmethod
-    def by_name(cls, key):
-        result = getattr(cls, key, None)
-        if isinstance(result, Verbosity):
-            return result
-        raise InvalidArgument('No such verbosity level %r' % (key,))
-
-
-Verbosity.quiet = Verbosity('quiet', 0)
-Verbosity.normal = Verbosity('normal', 1)
-Verbosity.verbose = Verbosity('verbose', 2)
-Verbosity.debug = Verbosity('debug', 3)
-Verbosity.all = [
-    Verbosity.quiet, Verbosity.normal, Verbosity.verbose, Verbosity.debug
-]
-
-
-ENVIRONMENT_VERBOSITY_OVERRIDE = os.getenv('HYPOTHESIS_VERBOSITY_LEVEL')
-
-if ENVIRONMENT_VERBOSITY_OVERRIDE:  # pragma: no cover
-    DEFAULT_VERBOSITY = Verbosity.by_name(ENVIRONMENT_VERBOSITY_OVERRIDE)
-else:
-    DEFAULT_VERBOSITY = Verbosity.normal
 
 settings.define_setting(
     'verbosity',
-    options=Verbosity.all,
-    default=DEFAULT_VERBOSITY,
+    options=tuple(Verbosity),
+    default=Verbosity._get_default(),
     description='Control the verbosity level of Hypothesis messages',
 )
 
@@ -684,11 +676,15 @@ Number of steps to run a stateful program for before giving up on it breaking.
 
 settings.define_setting(
     'perform_health_check',
-    default=True,
+    default=not_set,
     description=u"""
 If set to True, Hypothesis will run a preliminary health check before
 attempting to actually execute your test.
-"""
+""",
+    deprecation_message="""
+This setting is deprecated, as `perform_health_check=False` duplicates the
+effect of `suppress_health_check=HealthCheck.all()`.  Use that instead!
+""",
 )
 
 
